// Control.java - creates and maintains the PDBReader control panel

import shared.model.structure.*;
import shared.CoordinateLayout.*;
import shared.constants.*;
import shared.model.utils.Functions;
import shared.model.reader.FileHandler;

import java.awt.event.*;
import java.awt.*;

import javax.swing.event.*;
import javax.swing.*;

import javax.vecmath.*;
import java.util.*;

import java.io.*;

public class ConnectControl
{
        public ConnectControl()
        {
                //--- Initialise the control window ---//

                // Get the current look and feel
                try
		{
		    UIManager.setLookAndFeel(
				   UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {}

                // Create a JFrame
		JFrame controlWindow = new JFrame("ConnectEdit - Particle connection editor");
                controlWindow.setSize(800, 720);
		controlWindow.getContentPane().setLayout(new FlowLayout());
		controlWindow.addWindowListener(new WindowAdapter()
                {
                        public void windowClosing(WindowEvent e) {System.exit(0);}
                });


                // Create a JPanel for the objects
                JPanel mainPanel = new JPanel();
                mainPanel.setPreferredSize(new Dimension(800, 720));
                mainPanel.setLayout(new CoordinateLayout(new Dimension(16, 18), 2, 2));

                // create the graph
                final Graph graph = new Graph();

                // Create file choosers
                final JFileChooser fcOut = new JFileChooser(new File(Config.TEMPLATE_FILE_DIR));
                fcOut.setMultiSelectionEnabled(false);

                final JLabel outputLabel = new JLabel("None");

                // Create a button which starts the clustering process
                final JButton save = new JButton("Save");
                save.setEnabled(false);
                save.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e)
                        {
                                graph.save(fcOut.getSelectedFile());
                        }
                });

                // Create the output file selection button and action
                Action outputAction = new ConnectOutputAction(controlWindow, fcOut, outputLabel, save);
                JButton outputButton = new JButton(outputAction);

                // Create the vertical JSlider
                final JSlider valueBar = new JSlider(-100, 100, 0);
                valueBar.setMajorTickSpacing(10);
                valueBar.setMinorTickSpacing(1);
                valueBar.setPaintTicks(true);
                // label the slider
                Dictionary table = new Hashtable(); //valueBar.getLabelTable();
                JLabel label = new JLabel("-1");
                table.put(new Integer(valueBar.getMinimum()), label);
                label = new JLabel("-0.5");
                table.put(new Integer(valueBar.getMinimum() / 2), label);
                label = new JLabel("0");
                table.put(new Integer(0), label);
                label = new JLabel("0.5");
                table.put(new Integer(valueBar.getMaximum() / 2), label);
                label = new JLabel("1");
                table.put(new Integer(valueBar.getMaximum()), label);
                valueBar.setLabelTable(table);
                valueBar.setPaintLabels(true);
                valueBar.setSnapToTicks(true);


                // Create the horizontal JSlider
                final JSlider forceBar = new JSlider(JSlider.VERTICAL, -100, 100, 0);

                final JLabel coords = new JLabel("(0.0, 0.0)");

                // create a listener for the value bar
                valueBar.addChangeListener(new ChangeListener() {
                        public void stateChanged(ChangeEvent evt)
                        {
                                JSlider slider = (JSlider)evt.getSource();

                                if (!slider.getValueIsAdjusting())
                                {
                                        // Get new value
                                        graph.valueChanged(slider.getValue(), forceBar, coords);
                                }
                        }
                });

                forceBar.setMajorTickSpacing(10);
                forceBar.setMinorTickSpacing(1);
                forceBar.setPaintTicks(true);
                // label the slider
                Dictionary table2 = new Hashtable(); //forceBar.getLabelTable();
                label = new JLabel("-1");
                table2.put(new Integer(forceBar.getMinimum()), label);
                label = new JLabel("-0.5");
                table2.put(new Integer(forceBar.getMinimum() / 2), label);
                label = new JLabel("0");
                table2.put(new Integer(0), label);
                label = new JLabel("0.5");
                table2.put(new Integer(forceBar.getMaximum() / 2), label);
                label = new JLabel("1");
                table2.put(new Integer(forceBar.getMaximum()), label);
                forceBar.setLabelTable(table2);
                forceBar.setPaintLabels(true);
                forceBar.setSnapToTicks(true);
                // create a listener for the force bar
                forceBar.addChangeListener(new ChangeListener() {
                        public void stateChanged(ChangeEvent evt)
                        {
                                JSlider slider = (JSlider)evt.getSource();

                                if (!slider.getValueIsAdjusting())
                                {
                                        // Get new value
                                        graph.forceChanged(valueBar.getValue(), slider.getValue(), coords);
                                }
                        }
                });

                mainPanel.add(new JLabel("Output file: "), new Coordinate(2,1,4,2));
                mainPanel.add(outputLabel, new Coordinate(4,1,7,2));
                mainPanel.add(outputButton, new Coordinate(7,1,9,2));
		mainPanel.add(save, new Coordinate(9,1,11,2));

                mainPanel.add(valueBar, new Coordinate(1,15,14,17));
                mainPanel.add(forceBar, new Coordinate(14,3,16,15));
                mainPanel.add(graph, new Coordinate(1,3,14,15));
                mainPanel.add(coords, new Coordinate(13,2,16,3));

                controlWindow.getContentPane().add(mainPanel);

                controlWindow.setVisible(true);
        }

        public static void main(String[] args)
        {
                ConnectControl control = new ConnectControl();
        }
}

class Graph extends JPanel
{
        Vector points;

        int leftOffset = 9;
        int rightOffset = 9;
        int topOffset = 7;
        int bottomOffset = 16;

        int numLinesX = 21;
        int numLinesY = 21;
        int majorIntervalX = 5;
        int majorIntervalY = 5;

        Graphics graphics;


        Graph()
        {
                points = new Vector();
                points.addElement(new PointF(-1f,0f));
                points.addElement(new PointF(1f,0f));
        }

        void save(File file)
        {
                String filename = file.getName();
                String[] splitname = Functions.split('.', filename);
                String potentialId = splitname[0];

                // create a PotentialTemplate for the graph
                PotentialTemplate pt = new PotentialTemplate(potentialId);

                String[] forces = new String[points.size()];
                String[] values = new String[points.size()];
                for (int i=0; i<points.size(); i++)
                {
                        PointF p = (PointF) (points.elementAt(i));
                        forces[i] = "" + (p.getY());
                        values[i] = "" + (p.getX());
                }
                pt.setForces(forces);
                pt.setValues(values);

                Vector model = new Vector();
                model.addElement(pt);

                FileHandler fh = new FileHandler();
                fh.toFile(file.getPath(), model);

                System.out.println("forces - "+forces);
                System.out.println("values - "+values);
        }

        public void paint(Graphics g)
        {
                graphics = g;
                clear(g);
                drawGrid(g);
                drawLine(g, true);
        }

        void clear(Graphics g)
        {
                Dimension size = this.getSize();
                g.clearRect(0,0,(int) (size.getWidth()), (int) (size.getHeight()));
        }

        void valueChanged(int v, JSlider forceBar, JLabel coords)
        {
                float val = ((float) v / 100f);
                float newForce = 0f;
                PointF newPoint = null;
                // calculate an approximate value for the force at the given value and set the force slider knob to this force.
                int count = 0;
                while (count < points.size() && ((PointF)(points.elementAt(count))).getX() < val-0.005)
                {
                        count++;
                }
                if (count<points.size())
                {
                        newPoint = (PointF)(points.elementAt(count));
                        if (newPoint.getX() < val+0.005)
                        {
                                newForce = newPoint.getY();
                        }
                        else
                        {
                                newForce = -2f; // out of range for the graph
                        }
                }
                //forceBar.setValue((int) (newForce*100f));
                String forceText;
                if (newForce < -1.995f && newForce > -2.005f)
                {
                        forceText = "-";
                }
                else
                {
                        forceText = ""+newForce;
                }

                coords.setText("("+val+", "+forceText+")");
                repaint();
        }

        void forceChanged(int v, int f, JLabel coords)
        {
                drawLine(graphics, false);
                //drawGrid(graphics);

                float val = ((float) v / 100f);
                float force = ((float) f / 100f);

                // find the location of the point, if it exists, otherwise find the point at which it is to be inserted
                PointF newPoint = null;
                int count = 0;
                while (count < points.size() && ((PointF)(points.elementAt(count))).getX() < val-0.005)
                {
                        count++;
                }
                if (count<points.size())
                {
                        newPoint = (PointF)(points.elementAt(count));
                        if (newPoint.getX() < val+0.005f)
                        {
                                newPoint.setY(force);
                        }
                        else
                        {
                                newPoint = null;
                        }
                }

                if (newPoint == null)
                {
                        newPoint = new PointF(val, force);
                        points.insertElementAt(newPoint, count);
                }

                coords.setText("("+val+", "+force+")");
                repaint();
        }

        void drawLine(Graphics g, boolean erase)
        {
                Dimension size = this.getSize();
                int height = (int) size.getHeight()-bottomOffset;
                int width = (int) size.getWidth()-rightOffset;

                int xPixels = width - leftOffset;
                int yPixels = height; // - topOffset;
                float oneTickX = (float) xPixels / 200f;
                float oneTickY = (float) yPixels / 200f;


                g.setColor(new Color(64,64,255));

                for (int i=0; i<points.size()-1; i++)
                {
                        PointF start = (PointF) (points.elementAt(i));
                        PointF end = (PointF) (points.elementAt(i+1));

                        int x1 = leftOffset + (int)((float)xPixels * ((float)start.getX() + 1f)/2f);
                        int x2 = leftOffset + (int)((float)xPixels * ((float)end.getX() + 1f)/2f);
                        int y1 = topOffset + (int)((float)yPixels * (-(float)start.getY() + 1f)/2f);
                        int y2 = topOffset + (int)((float)yPixels * (-(float)end.getY() + 1f)/2f);

                        g.setColor(new Color(64,64,255));
                        g.drawLine(x1, y1, x2, y2);

                        g.setColor(new Color(64,255,64));
                        if (i == points.size()-2)
                        {
                                g.fillRect(x2-1, y2-1, 3, 3);
                        }
                        g.fillRect(x1-1, y1-1, 3, 3);
                }
        }

        void drawGrid(Graphics g)
        {
                Dimension size = this.getSize();
                int height = (int) size.getHeight()-bottomOffset;
                int width = (int) size.getWidth()-rightOffset;

                for (int i=0; i<=numLinesX; i++)
                {
                        if (i%majorIntervalX == 0)
                        {
                                g.setColor(new Color(0,0,0));
                        }
                        else
                        {
                                g.setColor(new Color(128,128,128));
                        }

                        g.drawLine(leftOffset, topOffset + i * height / (numLinesX-1), width, topOffset + i*height/(numLinesX-1));
                }

                for (int i=0; i<=numLinesY-1; i++)
                {
                        if (i%majorIntervalY == 0)
                        {
                                g.setColor(new Color(0,0,0));
                        }
                        else
                        {
                                g.setColor(new Color(128,128,128));
                        }
                        g.drawLine(leftOffset + i * (width - leftOffset) / (numLinesY-1), topOffset, leftOffset + i*(width-leftOffset)/(numLinesY-1), height+topOffset);
                }

        }
}

class PointF
{
        float x, y;

        PointF()
        {
                x=0f;
                y=0f;
        }

        PointF(float x, float y)
        {
                this.x = x;
                this.y = y;
        }

        float getX()
        {
                return x;
        }

        float getY()
        {
                return y;
        }

        void setX(float x)
        {
                this.x = x;
        }

        void setY(float y)
        {
                this.y = y;
        }
}

class ConnectOutputAction extends AbstractAction
{
        JFrame frame;
        JFileChooser chooser;
        File file = null;
        JLabel label;
        JButton saveButton;

        ConnectOutputAction(JFrame frm, JFileChooser ch, JLabel lab, JButton sb)
        {
            super("Change...");
            chooser = ch;
            frame = frm;
            label = lab;
            saveButton = sb;
        }

        public void actionPerformed(ActionEvent evt)
        {
            int result = chooser.showOpenDialog(frame);
            if (result == JFileChooser.APPROVE_OPTION)
            {
                file = chooser.getSelectedFile();
                //PDBReader.outputFile = file.getPath();
                label.setText(file.getName());
                saveButton.setEnabled(true);

            }
        }
}