// Control.java - creates and maintains the PDBReader control panel

import shared.model.structure.*;
import shared.CoordinateLayout.*;
import shared.constants.*;

import java.awt.event.*;
import java.awt.*;

import javax.swing.event.*;
import javax.swing.*;

import javax.vecmath.*;
import java.util.*;

import java.io.*;

public class PDBControl
{

        public PDBControl()
        {
                //--- Initialise the control window ---//

                // Get the current look and feel
                try
		{
		    UIManager.setLookAndFeel(
				   UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {}

                // Create a JFrame
		JFrame controlWindow = new JFrame("PDBReader - Abstract protein model creation");
                controlWindow.setSize(600, 300);
		controlWindow.getContentPane().setLayout(new FlowLayout());
		controlWindow.addWindowListener(new WindowAdapter()
                {
                        public void windowClosing(WindowEvent e) {System.exit(0);}
                });

                // Create a button which starts the clustering process
                JButton start = new JButton("Create");
                start.setEnabled(false);
                start.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e)
                        {
                                PDBReader.running = true;
                        }
                });

                // Create file choosers
                JFileChooser fcIn = new JFileChooser(new File(Config.PDB_FILE_DIR));
                JFileChooser fcOut = new JFileChooser(new File(Config.TEMPLATE_FILE_DIR));
                fcIn.setMultiSelectionEnabled(false);
                fcOut.setMultiSelectionEnabled(false);

                // Create the input file selection button and action
                JLabel inputLabel = new JLabel("None");
                Action inputAction = new InputAction(controlWindow, fcIn, inputLabel, start);
                JButton inputButton = new JButton(inputAction);

                // Create the output file selection button and action
                JLabel outputLabel = new JLabel("None");
                Action outputAction = new OutputAction(controlWindow, fcOut, outputLabel, start);
                JButton outputButton = new JButton(outputAction);

                // Add a text box for specifying the required number of clusters
                final JTextField clusterNumber = new JTextField(4);
                clusterNumber.setEnabled(false);
                clusterNumber.setText("10");
                clusterNumber.addKeyListener(new KeyListener() {
                        public void keyPressed(KeyEvent e)
                        {}
                        public void keyTyped(KeyEvent e)
                        {}

                        public void keyReleased(KeyEvent e)
                        {
                                String numString = ((JTextField) (e.getSource())).getText();
                                try
                                {
                                        int numClusters = Integer.parseInt(numString);
                                        PDBReader.numClusters = numClusters;
                                }
                                catch (NumberFormatException nf)
                                {}
                        }
                });

                // Create a menu of cluster methods
                JComboBox description = new JComboBox();
                description.setEditable(false);
                description.addItem("Atoms");
                description.addItem("Amino Acids");
                description.addItem("Clustered (atoms)");
                description.addItem("Clustered (amino acids)");
                description.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e)
                        {
                                int index = ((JComboBox) (e.getSource())).getSelectedIndex();
                                PDBReader.descriptionLevel = index;
                                if (index == 0 || index == 1)
                                {
                                        clusterNumber.setEnabled(false);
                                }
                                else
                                {
                                        clusterNumber.setEnabled(true);
                                }

                        }
                });

                // Create a JPanel for the objects
                JPanel mainPanel = new JPanel();
                mainPanel.setPreferredSize(new Dimension(600, 300));

                mainPanel.setLayout(new CoordinateLayout(new Dimension(10, 10), 2, 2));

                mainPanel.add(new JLabel("Input file: "), new Coordinate(2,1,4,2));
                mainPanel.add(inputLabel, new Coordinate(4,1,7,2));
                mainPanel.add(inputButton, new Coordinate(7,1,9,2));
                mainPanel.add(new JLabel("Output file: "), new Coordinate(2,2,4,3));
                mainPanel.add(outputLabel, new Coordinate(4,2,7,3));
                mainPanel.add(outputButton, new Coordinate(7,2,9,3));
                mainPanel.add(new JLabel("Cluster method:"), new Coordinate(2,4,5,5));
		mainPanel.add(description, new Coordinate(4,4,8,5));
                mainPanel.add(new JLabel("No. of Clusters: "), new Coordinate(2,5,5,6));
                mainPanel.add(clusterNumber, new Coordinate(4,5,8,6));
		mainPanel.add(start, new Coordinate(4,7,6,8));

                controlWindow.getContentPane().add(mainPanel);

                controlWindow.setVisible(true);
        }
}

class OutputAction extends AbstractAction
{
        JFrame frame;
        JFileChooser chooser;
        File file = null;
        JLabel label;
        JButton startButton;

        OutputAction(JFrame frm, JFileChooser ch, JLabel lab, JButton sb)
        {
            super("Change...");
            chooser = ch;
            frame = frm;
            label = lab;
            startButton = sb;
        }

        public void actionPerformed(ActionEvent evt)
        {
            int result = chooser.showOpenDialog(frame);
            if (result == JFileChooser.APPROVE_OPTION)
            {
                file = chooser.getSelectedFile();
                PDBReader.outputFile = file.getPath();
                label.setText(file.getName());

            }
        }
}

class InputAction extends AbstractAction
{
        JFrame frame;
        JFileChooser chooser;
        File file = null;
        JLabel label;
        JButton startButton;

        InputAction(JFrame frm, JFileChooser ch, JLabel lab, JButton sb)
        {
            super("Change...");
            chooser = ch;
            frame = frm;
            label = lab;
            startButton = sb;
        }

        public void actionPerformed(ActionEvent evt)
        {
            int result = chooser.showOpenDialog(frame);
            if (result == JFileChooser.APPROVE_OPTION)
            {
                file = chooser.getSelectedFile();
                PDBReader.inputFile = file.getPath();
                label.setText(file.getName());
                startButton.setEnabled(true);
            }
        }
}