import shared.model.structure.*;
import shared.constants.*;
import shared.model.display.*;
import shared.model.reader.*;
import shared.model.utils.Functions;
import shared.model.SimulationModel;

import java.io.*;
import java.util.*;
import javax.vecmath.*;

public class PDBReader
{
        final static int ATOMS = 0;
        final static int AMINO_ACIDS = 1;
        final static int ATOM_CLUSTERS = 2;
        final static int AMINO_ACID_CLUSTERS = 3;
        public static boolean running = false;
        public static int descriptionLevel = ATOMS;
        public static String inputFile = "";
        public static String outputFile = "";
        public static int numClusters = 10;

        public PDBReader()
        {
        }

        public static void main(String[] args) throws Exception
        {
                // Parameters
                float scale = Units.ANGSTROMS_PER_METRE;
                float displayScale = 1f / (10f * Units.ANGSTROMS_PER_METRE);

                // Start up the control panel
                PDBControl controlPanel = new PDBControl();


           while (true)
           {

                // Don't start until the user presses the start button
                while (running == false)
                {}

                // Create the simulationModel
                SimulationModel simulation = new SimulationModel();

                System.out.println("PDBReader - Automatic generation of simplified protein models from the PDB.");

                // Open the .pdb file for input
                FileReader fr = new FileReader(inputFile);
                BufferedReader br = new BufferedReader(fr);

                Vector atoms = new Vector();
                String particleXML = "";
                int atomID = 0;

                // Create a name for the object
                String[] pathArr = Functions.split('/', inputFile);
                String[] nameArr = Functions.split('.', pathArr[pathArr.length-1]);
                String objName = nameArr[0];

                // Create particle templates for the atoms
                // - Carbon
                ParticleTemplate carbon = new ParticleTemplate("C");
                carbon.setMass(Atoms.CARBON_MASS * scale);
                carbon.addState("State 0", Atoms.CARBON_COLOUR, 1f);
                float[] carbonRadius = {Atoms.CARBON_RADIUS * scale};
                carbon.setShape("sphere", carbonRadius);
                // - Nitrogen
                ParticleTemplate nitrogen = new ParticleTemplate("N");
                nitrogen.setMass(Atoms.NITROGEN_MASS * scale);
                nitrogen.addState("State 0", Atoms.NITROGEN_COLOUR, 1f);
                float[] nitrogenRadius = {Atoms.NITROGEN_RADIUS * scale};
                nitrogen.setShape("sphere", nitrogenRadius);
                // - Oxygen
                ParticleTemplate oxygen = new ParticleTemplate("O");
                oxygen.setMass(Atoms.OXYGEN_MASS * scale);
                oxygen.addState("State 0", Atoms.OXYGEN_COLOUR, 1f);
                float[] oxygenRadius = {Atoms.OXYGEN_RADIUS * scale};
                oxygen.setShape("sphere", oxygenRadius);
                // - Sulphur
                ParticleTemplate sulphur = new ParticleTemplate("S");
                sulphur.setMass(Atoms.SULPHUR_MASS * scale);
                sulphur.addState("State 0", Atoms.SULPHUR_COLOUR, 1f);
                float[] sulphurRadius = {Atoms.SULPHUR_RADIUS * scale};
                sulphur.setShape("sphere", sulphurRadius);

                // Add the atom templates to the creator
                simulation.addTemplate(carbon);
                simulation.addTemplate(nitrogen);
                simulation.addTemplate(oxygen);
                simulation.addTemplate(sulphur);

                // Create the containing object template
                SimObjectTemplate object = new SimObjectTemplate(objName);
                simulation.addTemplate(object);



                System.out.println("\nReading atom data");
                String pdbLine;
                Vector aminoAcidNum = new Vector();

                Vector particleTemplateList = new Vector();
                Vector particleCOMList = new Vector();
                Vector particleOrientList = new Vector();

                while((pdbLine = br.readLine()) != null)
                {
                        if((pdbLine.substring(0,4)).equals("ATOM"))
                        {
                                // Create a new particle object for the atom
                                String Sx = pdbLine.substring(31, 38);
                                String Sy = pdbLine.substring(39, 46);
                                String Sz = pdbLine.substring(47, 54);
                                Float Fx = Float.valueOf(Sx);
                                Float Fy = Float.valueOf(Sy);
                                Float Fz = Float.valueOf(Sz);

                                String atomName = pdbLine.substring(77, 78);
                                if (atomName == null)
                                {
                                     System.out.println("unknown atom - "+atomName);
                                     Runtime.getRuntime().exit(0);

                                }
                                particleTemplateList.addElement(atomName);
                                particleCOMList.addElement(new Vector3f(Fx.floatValue() * scale, Fy.floatValue() * scale, Fz.floatValue() * scale));
                                particleOrientList.addElement(new AxisAngle4f());

                                //Particle particle = new Particle();
                                //particle.setName(objName+":"+atomID+"-"+atomName);
                                //particle.setParticleTemplate((ParticleTemplate)(lookupTemplate.elementAt(templateIndex)));
                                //particle.setCentreOfMass(new Vector3f(Fx.floatValue() * scale, Fy.floatValue() * scale, Fz.floatValue() * scale));
                                //particleList.addElement(particle);

                                String AANum = pdbLine.substring(23, 26);
                                while (AANum.substring(0, 1).equals(" "))
                                {
                                        AANum = AANum.substring(1);
                                }
                                aminoAcidNum.addElement(new Integer(Integer.parseInt(AANum)));


                                atomID++;
                                System.out.print(".");
                        }
                }

                // Adjust the particles so that their centre of mass is at (0, 0, 0)
                float massTimesDistX = 0f;
                float massTimesDistY = 0f;
                float massTimesDistZ = 0f;
                float totalMass = 0f;

                for (int i=0; i<particleTemplateList.size(); i++)
                {
                        Vector3f pos = (Vector3f) (particleCOMList.elementAt(i));
                        float mass = ((ParticleTemplate)(simulation.getTemplate((String)(particleTemplateList.elementAt(i))))).getMass();

                        massTimesDistX += mass * pos.x;
                        massTimesDistY += mass * pos.y;
                        massTimesDistZ += mass * pos.z;
                        totalMass += mass;
                }

                float transX =  - (massTimesDistX / totalMass);
                float transY =  - (massTimesDistY / totalMass);
                float transZ =  - (massTimesDistZ / totalMass);
                Vector3f translation = new Vector3f(transX, transY, transZ);

                for (int i=0; i<particleCOMList.size(); i++)
                {
                        Vector3f com = (Vector3f) (particleCOMList.elementAt(i));
                        com.x += translation.x;
                        com.y += translation.y;
                        com.z += translation.z;
                }

                System.out.println("\nComplete.");

                Vector AATemplates = new Vector();
                // Cluster the atoms by amino acid, if required
                if (descriptionLevel == AMINO_ACIDS || descriptionLevel == AMINO_ACID_CLUSTERS)
                {
                      Vector aminoParticleList = new Vector();
                      int aminoID = 0;

                      System.out.println("\nCreating Amino Acids");

                      int numAminos = 0;
                      while (particleTemplateList.size() > 0 + numAminos)
                      {
                                int currentAcid = ((Integer) (aminoAcidNum.elementAt(0))).intValue();
                                massTimesDistX = 0f;
                                massTimesDistY = 0f;
                                massTimesDistZ = 0f;
                                totalMass = 0f;
                                //Vector atomsInAcid = new Vector();
                                float minX = Float.MAX_VALUE;
                                float minY = Float.MAX_VALUE;
                                float minZ = Float.MAX_VALUE;
                                float maxX = -Float.MAX_VALUE;
                                float maxY = -Float.MAX_VALUE;
                                float maxZ = -Float.MAX_VALUE;

                                // Replace atoms in a single amino acid by a single particle
                                while (aminoAcidNum.size() > 0 && ((Integer) (aminoAcidNum.elementAt(0))).intValue() == currentAcid)
                                {
                                        //atomsInAcid.addElement(p);
                                        Vector3f pos = (Vector3f)(particleCOMList.elementAt(0));
                                        float mass = ((ParticleTemplate)(simulation.getTemplate(
                                                                (String)(particleTemplateList.elementAt(0))))).getMass();

                                        massTimesDistX += mass * pos.x;
                                        massTimesDistY += mass * pos.y;
                                        massTimesDistZ += mass * pos.z;
                                        totalMass += mass;

                                        if (pos.x > maxX) {maxX = pos.x;}
                                        if (pos.y > maxY) {maxY = pos.y;}
                                        if (pos.z > maxZ) {maxZ = pos.z;}
                                        if (pos.x < minX) {minX = pos.x;}
                                        if (pos.y < minY) {minY = pos.y;}
                                        if (pos.z < minZ) {minZ = pos.z;}

                                        particleTemplateList.removeElementAt(0);
                                        particleCOMList.removeElementAt(0);
                                        particleOrientList.removeElementAt(0);
                                        aminoAcidNum.removeElementAt(0);
                                }

                                // Create a particle template and particle for the amino acid
                                Vector3f centreOfMass = new Vector3f(massTimesDistX / totalMass,
                                                                     massTimesDistY / totalMass,
                                                                     massTimesDistZ / totalMass);

                                float radius = ((maxX - minX) + (maxY - minY) + (maxZ - minZ))/3f;

                                // template
                                ParticleTemplate aminoTemplate = new ParticleTemplate("amino"+currentAcid);
                                aminoTemplate.setMass(totalMass);
                                aminoTemplate.addState("state 0", new Color3f(1f, 0.2f, 1f), 1f);
                                float[] aminoRadius = {radius};
                                aminoTemplate.setShape("sphere", aminoRadius);
                                AATemplates.addElement(aminoTemplate);
                                simulation.addTemplate(aminoTemplate);

                                //// particle
                                //Particle aminoParticle = new Particle();
                                //aminoParticle.setName(objName+":"+(aminoID++) + "-" + aminoTemplate.getName());
                                //aminoParticle.setParticleTemplate(aminoTemplate);
                                //aminoParticle.setCentreOfMass(centreOfMass);
                                particleTemplateList.addElement("amino"+currentAcid);
                                particleCOMList.addElement(centreOfMass);
                                particleOrientList.addElement(new AxisAngle4f());
                                numAminos++;

                                System.out.print(".");

                      }
                      System.out.println("\nComplete.");
                }

                // Cluster the atoms or amino acids into a given number of particles if required
                if (descriptionLevel == ATOM_CLUSTERS || descriptionLevel == AMINO_ACID_CLUSTERS)
                {
                        System.out.println("\nClustering into "+numClusters+" particles:");
                        int clusterID = 0;

                        Vector clusterTemplateList = new Vector();
                        //simulationModel.clear();

                        while (particleTemplateList.size() > numClusters)
                        {
                                // Identify the two closest clusters... very naively
                                float minDistSquared = Float.MAX_VALUE;
                                int c1 = 0;
                                int c2 = 0;

                                for (int i=0; i<particleTemplateList.size(); i++)
                                {
                                        for (int j = 0; j<i; j++)
                                        {
                                                Vector3f c1Pos = (Vector3f) (particleCOMList.elementAt(i));
                                                Vector3f c2Pos = (Vector3f) (particleCOMList.elementAt(j));

                                                // compare squares to avoid sqrt operations
                                                float deltaX = c1Pos.x - c2Pos.x;
                                                float deltaY = c1Pos.y - c2Pos.y;
                                                float deltaZ = c1Pos.z - c2Pos.z;
                                                float distSquared = deltaX*deltaX + deltaY*deltaY + deltaZ*deltaZ;
                                                if (distSquared < minDistSquared)
                                                {
                                                        minDistSquared = distSquared;
                                                        c1 = i;
                                                        c2 = j;
                                                }
                                        }
                                }

                                String cluster1 = (String) (particleTemplateList.elementAt(c1));
                                String cluster2 = (String) (particleTemplateList.elementAt(c2));
                                Vector3f pos1 = (Vector3f) (particleCOMList.elementAt(c1));
                                Vector3f pos2 = (Vector3f) (particleCOMList.elementAt(c2));

                                particleTemplateList.removeElementAt(c1);
                                particleTemplateList.removeElementAt(c2);
                                particleCOMList.removeElementAt(c1);
                                particleCOMList.removeElementAt(c2);
                                particleOrientList.removeElementAt(c1);
                                particleOrientList.removeElementAt(c2);

                                // Combine the clusters and create a new particle and template
                                float mass1 = ((ParticleTemplate)(simulation.getTemplate(cluster1))).getMass();
                                float mass2 = ((ParticleTemplate)(simulation.getTemplate(cluster2))).getMass();
                                float newMass = mass1 + mass2;

                                Vector3f newPos = new Vector3f((mass1*pos1.x + mass2*pos2.x)/newMass,
                                                               (mass1*pos1.y + mass2*pos2.y)/newMass,
                                                               (mass1*pos1.z + mass2*pos2.z)/newMass);

                                float newRadius = 0.5f*(float) Math.sqrt(minDistSquared)
                                                + 0.8f*((ParticleTemplate)(simulation.getTemplate(cluster1))).getShapeRadius()
                                                + 0.8f*((ParticleTemplate)(simulation.getTemplate(cluster2))).getShapeRadius();

                                // template
                                ParticleTemplate clusterTemplate = new ParticleTemplate("cluster-"+clusterID);
                                clusterTemplate.setMass(newMass);
                                clusterTemplate.addState("state 0", new Color3f(0.2f, 1f, 0.2f), 1f);
                                float[] clusterRadius = {newRadius};
                                clusterTemplate.setShape("sphere", clusterRadius);
                                clusterTemplateList.add(clusterTemplate);
                                simulation.addTemplate(clusterTemplate);

                                particleTemplateList.addElement("cluster-"+clusterID);
                                particleCOMList.addElement(newPos);
                                particleOrientList.addElement(new AxisAngle4f());
                                clusterID++;

                                System.out.print(".");
                        }

                        String cName = "";
                        String pName = "";

                        //simulationModel.clear();

                        // determine which cluster templates are required
                        int plSize = particleTemplateList.size();
                        for (int i=0; i<plSize; i++)
                        {
                                for (int j=0; j<clusterTemplateList.size(); j++)
                                {
                                        cName = ((ParticleTemplate)(clusterTemplateList.elementAt(j))).getId();
                                        pName = (String)(particleTemplateList.elementAt(i));
                                        if (cName.equals(pName))
                                        {
                                                simulation.addTemplate((ParticleTemplate) (clusterTemplateList.elementAt(j)));
                                        }
                                }

                                for (int j=0; j<AATemplates.size(); j++)
                                {
                                        cName = ((ParticleTemplate)(AATemplates.elementAt(j))).getId();
                                        pName = (String)(particleTemplateList.elementAt(i));
                                        if (cName.equals(pName))
                                        {
                                                simulation.addTemplate((ParticleTemplate) (AATemplates.elementAt(j)));
                                        }
                                }
                        }
                        System.out.println("\nComplete.");
                }

                // Create object template
                System.out.println("\nCreate object:");

                String[] partTemps = new String[particleTemplateList.size()];
                Vector3f[] partCOMs = new Vector3f[particleTemplateList.size()];
                AxisAngle4f[] partOrients = new AxisAngle4f[particleTemplateList.size()];
                for (int i=0; i<particleTemplateList.size(); i++)
                {
                        partTemps[i] = (String) (particleTemplateList.elementAt(i));
                        partCOMs[i] = (Vector3f) (particleCOMList.elementAt(i));
                        partOrients[i] = (AxisAngle4f) (particleOrientList.elementAt(i));
                }
                object.setParticles(partTemps, partCOMs, partOrients);

                simulation.addTemplate(object);

                SimObject instance = new SimObject("inst");
                instance.setTemplateId(object.getId());
                instance.setCentreOfMass(new Vector3f());
                instance.setOrientation(new AxisAngle4f());

                simulation.createObjectInstance(instance);

                System.out.println("Complete.");

                System.out.println("\nRendering Model:");

                // Display the object
                simulation.setSimulationSize(10f / displayScale);
                simulation.display();
                System.out.println("Complete.");

                // Output the file if required.
                if (!(outputFile.equals("")))
                {
                        System.out.println("\nWriting file:");

                        simulation.writeTemplateFile(outputFile);
                        System.out.println("Complete.");
                }

           running = false;
           }
        }
}






















