package shared.model.reader.xmlobjects;

import shared.model.structure.*;

import java.util.*;

import javax.vecmath.*;

public class XMLParticleTemplate
{
        public String id;
        public float mass;

        public String shape;
        public neuroml.core.List dimensions = new neuroml.core.List();

        public neuroml.core.List stateName = new neuroml.core.List();
        public neuroml.core.List stateColourR = new neuroml.core.List();
        public neuroml.core.List stateColourG = new neuroml.core.List();
        public neuroml.core.List stateColourB = new neuroml.core.List();
        public neuroml.core.List stateAlpha = new neuroml.core.List();

        public XMLParticleTemplate()
        {
        }

        public XMLParticleTemplate(ParticleTemplate pt)
        {
                id = pt.getId();
                mass = pt.getMass();

                shape = pt.getShapeName();
                float[] dims = pt.getShapeDimensions();
                for (int i=0; i<dims.length; i++)
                {
                        dimensions.add(new Float(dims[i]));
                }

                for (int i=0; i<pt.numStates(); i++)
                {
                        stateName.add(pt.getStateName(i));

                        Color3f col = pt.getStateColour(i);
                        stateColourR.add(new Float(col.x));
                        stateColourG.add(new Float(col.y));
                        stateColourB.add(new Float(col.z));

                        stateAlpha.add(new Float(pt.getStateAlpha(i)));
                }
        }

        public ParticleTemplate getObject()
        {
                ParticleTemplate pt = new ParticleTemplate(id);
                pt.setMass(mass);

                Object[] dimObjs = dimensions.toArray();
                float[] dims = new float[dimObjs.length];
                for (int i=0; i<dims.length; i++)
                {
                        dims[i] = ((Float) dimObjs[i]).floatValue();
                }

                pt.setShape(shape, dims);

                Object[] names = stateName.toArray();
                Object[] red = stateColourR.toArray();
                Object[] green = stateColourG.toArray();
                Object[] blue = stateColourB.toArray();
                Object[] alpha = stateAlpha.toArray();

                for (int i=0; i<stateName.size(); i++)
                {
                        String n = (String) names[i];
                        Color3f c = new Color3f(((Float) red[i]).floatValue(), ((Float) green[i]).floatValue(), ((Float) blue[i]).floatValue());
                        float a = ((Float) alpha[i]).floatValue();

                        pt.addState(n, c, a);
                }

                return pt;

        }
}